@@ -1,23 +1,31 @@
 import decimal
 import json
+import logging
 import logging.config
 import logging.handlers
 import os
 import warnings
 from abc import abstractmethod
-from typing import Optional, List
+from contextlib import contextmanager
+from copy import deepcopy
+from typing import Optional, List, Union
 
-from pydantic import BaseSettings, Field, BaseConfig, Extra, validator
+import redis
+from elasticsearch import Elasticsearch
+from pydantic import BaseSettings, Field, BaseConfig, Extra
 from pydantic.typing import StrPath
+from sqlalchemy import create_engine
+from sqlalchemy.orm import sessionmaker, scoped_session, Session
 
 from joeypan import RESOURCE_JOEYPAN_DIR
 
 # disable warning: pydantic set alias_namefor klass -> class
 warnings.filterwarnings("ignore", category=FutureWarning)
+LOG = logging.getLogger(__name__)
 
 
 class InjectSettings:
-    __DEFAULT_LOG_DIR = os.path.join(os.getcwd(), "logs")
+    __DEFAULT_LOG_DIR = os.environ.get("LOG_DIR") or os.path.join(os.getcwd(), "logs")
 
     @property
     def default_log_dir(self) -> str:
@@ -70,28 +78,74 @@ class Server(AbstractSettings):
 
 
 class Datasource(AbstractSettings):
+    SCOPED_SESSION: scoped_session = None
     url: str = Field(
         default="mysql+pymysql://user:password@localhost:3306/foo?charset=utf8mb4")
+    echo: bool = True
+    echo_pool: bool = False
+    pool_size: int = 10
+    pool_timeout: int = 3
+    pool_recycle: int = 10
+    pool_use_lifo: bool = True
+    query_cache_size: int = 500
+
+    @property
+    def __session(self) -> scoped_session:
+        if self.SCOPED_SESSION:
+            return self.SCOPED_SESSION
+        cfg = self.dict()
+        cfg.pop("SCOPED_SESSION", None)
+        url = cfg.pop("url", None)
+        engine = create_engine(url, **cfg)
+        session_factory = sessionmaker(engine)
+        self.SCOPED_SESSION = scoped_session(session_factory)
+        return self.SCOPED_SESSION
+
+    @property
+    @contextmanager
+    def session(self) -> Session:
+        session = self.__session()
+        try:
+            yield session
+            session.commit()
+        except Exception as e:
+            LOG.error(f"commit failed: {e}")
+            session.rollback()
+            raise e
+        finally:
+            session.close()
 
 
 class JedisPool(AbstractSettings):
-    max_active: int = Field(default=8)
-    max_wait: int = Field(default=-1)
-    max_idle: int = Field(default=8)
-    min_idle: int = Field(default=0)
-
-
-class Jedis(AbstractSettings):
-    pool: JedisPool = Field(default_factory=JedisPool)
-    timeout: int = Field(default=0)
+    pass
 
 
 class Redis(AbstractSettings):
-    database: int = Field(default=0)
+    POOL: Union[redis.Redis, redis.RedisCluster] = None
+    db: int = Field(default=0)
     host: str = Field(default="localhost")
     port: int = Field(default=6379)
-    password: str = None
-    jedis: Jedis = Field(default_factory=Jedis)
+    password: str = None,
+    startup_nodes: list = None
+    jedis: JedisPool = Field(default_factory=JedisPool)
+
+    @property
+    def session(self):
+        if not self.POOL:
+            if self.startup_nodes:
+                kwargs = self.jedis.dict()
+                klass = redis.RedisCluster
+            else:
+                kwargs = {
+                    "db": self.db,
+                    "host": self.host,
+                    "port": self.port,
+                    "password": self.password
+                }
+                kwargs.update(self.jedis.dict())
+                klass = redis.Redis
+            self.POOL = klass(**kwargs)
+        return self.POOL
 
 
 class LogFormat(AbstractSettings):
@@ -127,31 +181,21 @@ class LogFileHandler(LogHandler):
     klass: str = Field(default="logging.handlers.RotatingFileHandler")
     level: int = Field(default=logging.DEBUG)
     formatter: str = Field(default="upgrade")
-    filename: StrPath = Field(default=os.path.join(
-        inject_settings.default_log_dir, "all.log"))
+    filename: StrPath = Field(default="all.log")
     maxBytes: int = Field(default=4194304)  # 4MB
     backupCount: int = Field(default=50)  # 保留多少个log文件
     encoding: str = Field(default="utf8")
 
-    @validator("filename")
-    def reset_filename(cls, filename):
-        return os.path.join(inject_settings.default_log_dir, "all.log")
-
 
 class LogServerHandler(LogFileHandler):
     klass: str = Field(default="logging.handlers.RotatingFileHandler")
     level: int = Field(default=logging.DEBUG)
     formatter: str = Field(default="simple")
-    filename: StrPath = Field(default=os.path.join(
-        inject_settings.default_log_dir, "server.log"))
+    filename: StrPath = Field(default="server.log")
     maxBytes: int = Field(default=4194304)  # 4MB
     backupCount: int = Field(default=20)  # 保留多少个log文件
     encoding: str = Field(default="utf8")
 
-    @validator("filename")
-    def reset_filename(cls, filename):
-        return os.path.join(inject_settings.default_log_dir, "server.log")
-
 
 class LogHandlers(AbstractSettings):
     console_handler: LogHandler = Field(default_factory=LogConsoleHandler)
@@ -185,11 +229,24 @@ class Logger(AbstractSettings):
     loggers: LoggerInstances = Field(default_factory=LoggerInstances)
 
 
+class ES(AbstractSettings):
+    INSTANCE: Elasticsearch = None
+    hosts: List[dict] = Field(default=[{"host": "localhost", "port": 9200}])
+    kwargs: dict = Field(default_factory=dict)
+
+    @property
+    def session(self) -> Elasticsearch:
+        if not self.INSTANCE:
+            self.INSTANCE = Elasticsearch(hosts=self.hosts, **self.kwargs)
+        return self.INSTANCE
+
+
 class DefaultSettings(AbstractSettings):
     server: Server = Field(default_factory=Server)
     datasource: Datasource = Field(default_factory=Datasource)
     redis: Redis = Field(default_factory=Redis)
     log: Logger = Field(default_factory=Logger)
+    es: ES = Field(default_factory=ES)
 
 
 class UserSettings(DefaultSettings):
@@ -250,7 +307,28 @@ class SettingsFactory:
 
     @classmethod
     def setup_log(cls, cfg: Logger):
-        logging.config.dictConfig(cfg.dict())
+        def update_log_path(file_name):
+            if file_name and os.path.basename(file_name) == file_name:
+                return os.path.join(inject_settings.default_log_dir, file_name)
+            return file_name
+
+        def update_dict(data: dict):
+            if data.get("filename"):
+                data["filename"] = update_log_path(data.get("filename"))
+            return data
+
+        temp_cfg = deepcopy(cfg)
+        temp_handlers = {}
+        for name, handler in cfg.handlers:
+            if isinstance(name, (str, bytes)):
+                if isinstance(handler, LogHandler):
+                    if hasattr(handler, "filename"):
+                        handler.filename = os.path.join(inject_settings.default_log_dir, handler.filename)
+                    temp_handlers[name] = handler
+                elif isinstance(handler, dict):
+                    temp_handlers[name] = update_dict(handler)
+        temp_cfg.handlers = temp_handlers
+        logging.config.dictConfig(temp_cfg.dict())
 
     @classmethod
     def default(cls):
