@@ -14,6 +14,9 @@ class AbstractException(Exception):
     def dict(self):
         return {"code": self.code, "msg": self.msg}
 
+    def __bool__(self):
+        return 200 <= self.code < 400
+
 
 if __name__ == '__main__':
     try:
